# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-05 14:57
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('customer_id', models.CharField(max_length=256)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subscription_id', models.CharField(max_length=255)),
                ('verbose_name', models.CharField(max_length=255)),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subscription', to='lp_stripe.Customer')),
            ],
        ),
    ]



from django.contrib.auth import get_user_model
from rest_framework import serializers
from .models import Subscription

User = get_user_model()


class SubscriptionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Subscription
        exclude = ['customer']


class SubscriptionCreateSerializer(serializers.Serializer):
    source = serializers.CharField()
    plan = serializers.CharField(required=True)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-05 22:54
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('lp_stripe', '0002_auto_20170905_1802'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='customer', to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test import TestCase, override_settings
from parameterized import parameterized
from rest_framework import status
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient

User = get_user_model()


class SubscribeTestCase(TestCase):
    fixtures = ['lp_stripe_users']

    def setUp(self):
        self.client = APIClient()

    def authenticate(self, user=None):
        self.client.credentials()
        if user:
            token, created = Token.objects.get_or_create(user=user)
            self.client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)

    @staticmethod
    def modify_data(data={}, exclude=[], injections={}):
        data = {**data, **injections}
        for field in exclude:
            data.pop(field, None)

        return data

    @staticmethod
    def get_default_user():
        return User.objects.get(username='derek.jeter')

    @staticmethod
    def get_admin_user():
        return User.objects.get(username='ken.griffey')

    def subscribe(self, **kwargs):
        return self.client.post(
            reverse('lp_stripe_subscriptioncreate'),
            format='json',
            **kwargs
        )


@override_settings(STRIPE_TEST_API_KEY='sk_test_Eo1tQw3uuPrl8uG6kUZ8rjcl')
class SubscribeTestCase(SubscribeTestCase):
    PERMISSIONS = [
        ('Anonymous User', None, status.HTTP_401_UNAUTHORIZED),
        ('Authenticated User', SubscribeTestCase.get_default_user(), status.HTTP_201_CREATED),
        ('Authenticated Admin User', SubscribeTestCase.get_admin_user(), status.HTTP_201_CREATED)
    ]
    REQUIRED_FIELDS = [
        ('source', 'source'),
        ('plan', 'plan')
    ]

    def setUp(self):
        super(SubscribeTestCase, self).setUp()
        self.user = self.get_default_user()
        self.data = {
            'source': 'tok_amex',
            'plan': 'gold'
        }

    @parameterized.expand(PERMISSIONS)
    def test_permission(self, _, user, expected):
        self.authenticate(user=user)
        response = self.subscribe(data=self.data)
        self.assertEqual(response.status_code, expected)

    @parameterized.expand(REQUIRED_FIELDS)
    def test_required_field(self, _, field):
        self.authenticate(user=self.get_default_user())
        data = self.modify_data(self.data, exclude=[field])
        response = self.subscribe(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_can_subscribe(self):
        self.authenticate(user=self.user)
        response = self.subscribe(data=self.data)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, response.data)

    @override_settings(STRIPE_TEST_API_KEY=None)
    def test_missing_api_configuration(self):
        self.authenticate(user=self.user)
        response = self.subscribe(data=self.data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)



from django.contrib.auth import get_user_model
from django.db import models
from .client import Client


User = get_user_model()


class Customer(models.Model):
    user = models.ForeignKey(User, related_name='customer')
    stripe_id = models.CharField(max_length=256)

    def __init__(self, *args, **kwargs):
        self.client = Client()
        super(Customer, self).__init__(*args, **kwargs)


class Subscription(models.Model):
    customer = models.ForeignKey(Customer, related_name='subscription')
    stripe_id = models.CharField(max_length=255)
    verbose_name = models.CharField(max_length=255)

    def __init__(self, *args, **kwargs):
        self.client = Client()
        super(Subscription, self).__init__(*args, **kwargs)



from django.conf.urls import url
from rest_framework.urlpatterns import format_suffix_patterns
from .views import SubscriptionCreateView

urlpatterns = {
    url(r'^stripe/subscription', SubscriptionCreateView.as_view(), name='lp_stripe_subscriptioncreate')
}

urlpatterns = format_suffix_patterns(urlpatterns)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-05 18:02
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('lp_stripe', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='customer',
            old_name='customer_id',
            new_name='stripe_id',
        ),
        migrations.RenameField(
            model_name='subscription',
            old_name='subscription_id',
            new_name='stripe_id',
        ),
    ]



from parameterized import parameterized

from django.test import TestCase, override_settings
from .conf import Settings
from .client import Client


class SettingsTestCase(TestCase):
    def setUp(self):
        self.settings = Settings()

    def test_default_test_mode(self):
        self.assertEqual(self.settings.mode, Settings.MODE_TEST)

    @override_settings(STRIPE_MODE=Settings.MODE_LIVE)
    def test_can_set_mode(self):
        self.assertEqual(self.settings.mode, Settings.MODE_LIVE)

    @override_settings(STRIPE_TEST_API_KEY='testapikey', STRIPE_API_KEY='apikey')
    def test_fetch_test_mode_api_key(self):
        self.assertEqual(self.settings.api_key, 'testapikey')

    @override_settings(STRIPE_MODE=Settings.MODE_LIVE, STRIPE_TEST_API_KEY='testapikey', STRIPE_API_KEY='apikey')
    def test_fetch_live_mode_api_key(self):
        self.assertEqual(self.settings.api_key, 'apikey')


@override_settings(STRIPE_TEST_API_KEY='sk_test_Eo1tQw3uuPrl8uG6kUZ8rjcl')
class ClientTestCase(TestCase):
    def setUp(self):
        self.settings = Settings()
        self.client = Client(api_key=self.settings.api_key)
        self.customer_data = {
            'description': 'Test Customer from django-rest-stripe UnitTest',
            'source': 'tok_amex'
        }
        self.created_customers = []

    def tearDown(self):
        for customer in self.created_customers:
            customer.delete()

    def test_can_create_customer(self):
        try:
            customer = self.client.create_customer(**self.customer_data)
            self.created_customers.append(customer)
        except Exception as e:
            self.fail('Exception was raised: %s' % e)

    def test_can_subscribe_customer(self):
        customer = self.client.create_customer(**self.customer_data)
        self.created_customers.append(customer)
        try:
            subscription = self.client.create_subscription(customer=customer.stripe_id, plan='gold')
        except Exception as e:
            self.fail('Exception was raised: %s' % e)




from setuptools import setup
setup(
  name='lp_stripe',
  packages=['lp_stripe'],
  package_data={'lp_stripe': ['migrations/*', 'fixtures/*']},
  version='0.1.4',
  description='REST Framework Stripe Payments',
  author='Jim Simon',
  author_email='hello@launchpeer.com',
  url='https://github.com/Launchpeer/django-rest-stripe',
  download_url='https://github.com/Launchpeer/django-rest-stripe/archive/master.tar.gz',
  keywords=[],
  classifiers=[],
)


import stripe
from .conf import Settings


class Client(object):
    def __init__(self, api_key=None):
        self.settings = Settings()
        self.stripe = stripe
        self.stripe.api_key = api_key if api_key else self.settings.api_key

    def create_customer(self, description=None, source=None):
        if not description or not source:
            raise Exception('description and source fields are required')

        return self.stripe.Customer.create(description=description, source=source)

    def create_subscription(self, customer=None, plan=None):
        if not customer or not plan:
            raise Exception('customer and plan fields are required')

        return self.stripe.Subscription.create(customer=customer, items=[{'plan': plan}])



from rest_framework import generics, permissions, status
from rest_framework.response import Response
from .client import Client
from .models import Subscription, Customer
from .serializers import SubscriptionCreateSerializer


class SubscriptionCreateView(generics.CreateAPIView):
    serializer_class = SubscriptionCreateSerializer
    queryset = Subscription.objects.all()
    permission_classes = [
        permissions.IsAuthenticated
    ]

    def create(self, request, *args, **kwargs):
        source = request.data.get('source')
        plan = request.data.get('plan')
        user = self.request.user

        client = Client()
        try:
            stripe_customer = client.create_customer(description='Customer for %s' % user.username, source=source)
        except Exception as e:
            return Response(status=status.HTTP_400_BAD_REQUEST, data={'error': 'Could not create Stripe Customer: %s' % e})

        try:
            stripe_subscription = client.create_subscription(customer=stripe_customer['id'], plan=plan)
        except Exception as e:
            return Response(status=status.HTTP_400_BAD_REQUEST, data={'error': 'Could not create Stripe Subscription: %s' % e})

        customer = Customer.objects.create(user=user, stripe_id=stripe_customer['id'])
        subscription = Subscription.objects.create(customer=customer, stripe_id=stripe_subscription['id'], verbose_name=stripe_subscription['plan']['id'])

        return Response(status=status.HTTP_201_CREATED)



from django.conf import settings


class Settings(object):
    MODE_TEST = 1
    MODE_LIVE = 2
    
    @property
    def mode(self):
        return getattr(settings, 'STRIPE_MODE', self.MODE_TEST)

    @property
    def is_live(self):
        return self.mode is self.MODE_LIVE

    @property
    def api_key(self):
        test_api_key = getattr(settings, 'STRIPE_TEST_API_KEY', None)
        api_key = getattr(settings, 'STRIPE_API_KEY', None)

        return api_key if self.is_live else test_api_key
